@@ -46,6 +46,10 @@ gdal.UseExceptions()  # Enable exception support
 ogr.UseExceptions()  # Enable exception support
 osr.UseExceptions()  # Enable exception support
 
+# Version of the mesh that is produced
+MESH_MAJOR = "1"
+MESH_MINOR = "0"
+MESH_PATCH = "0"
 
 def main():
     # load user configurable parameters here
@@ -256,6 +260,17 @@ def main():
     if hasattr(X, 'extent'):
         extent = X.extent
 
+    #Clip to a shape file
+    clip_to_shp = None
+    if hasattr(X, 'clip_to_shp'):
+        clip_to_shp = X.clip_to_shp
+
+        if not os.path.exists(clip_to_shp):
+            raise Exception(f'Clipping shape file is not valid. Path given was\n {clip_to_shp}')
+
+    if clip_to_shp and extent:
+        raise Exception('Cannot specify both extent and a shape file to clip to')
+
     nworkers = os.cpu_count() or 1
 
     # on linux we can ensure that we respect cpu affinity
@@ -314,10 +329,15 @@ def main():
 
     base_dir = user_output_dir + base_name + os.path.sep
 
+    if not os.path.isdir(base_dir) and reuse_mesh:
+        raise Exception(f"Using reuse_mesh=True, however the directory {base_dir} does not exist")
+
     # Delete previous dir (if exists)
     if os.path.isdir(base_dir) and not reuse_mesh:
         shutil.rmtree(base_dir, ignore_errors=True)
 
+
+
     # these have to be separate ifs for the logic to work correctly
     if not reuse_mesh:
         # make new output dir
@@ -366,9 +386,13 @@ def main():
         ext_str = ' -te %s %s %s %s -te_srs \"%s\" ' % (extent[0], extent[1], extent[2], extent[3], srs.ExportToProj4())
         src_ds = None
 
-    e = 'GDAL_CACHEMAX=\"5%%\" %sgdalwarp %s %s -ot Float32 -multi -overwrite -dstnodata -9999 -t_srs \"%s\"' + ext_str
+    cutline_cmd = ''
+    if clip_to_shp:
+        cutline_cmd = f' -cutline {clip_to_shp} -crop_to_cutline '
+
+    e = 'GDAL_CACHEMAX=\"5%%\" %sgdalwarp %s %s -ot Float32 -multi -overwrite -dstnodata -9999 %s -t_srs \"%s\"' + ext_str
     subprocess.check_call([e % (gdal_prefix,
-                                dem_filename, base_dir + base_name + output_file_name, srs_out.ExportToProj4())],
+                                dem_filename, base_dir + base_name + output_file_name, cutline_cmd, srs_out.ExportToProj4())],
                           shell=True)
 
     if fill_holes:
@@ -923,6 +947,8 @@ def main():
         for key, data in t.items():
             ics[key].append(data)
 
+    mesh['mesh']['version'] = f'{MESH_MAJOR}.{MESH_MINOR}.{MESH_PATCH}'
+
     print('Total time took %s s' % str(round(time.perf_counter() - start_time, 2)))
 
     if output_write_vtu:
@@ -1312,7 +1338,7 @@ def rasterize_elem(rds, mem_layer, aggMethod, srs, new_gt, src_offset):
             output = raster.GetNoDataValue()
     else:
         if aggMethod == 'mode':
-            vals, counts = np.unique(src_array, return_counts=True)
+            vals, counts = np.unique( src_array[~np.isnan(src_array)], return_counts=True)
             output = float(vals[np.argmax(counts)])
 
         elif aggMethod == 'mean':
